<?php

/**
 * Ceon URI Mapping Configuration Utility.
 *
 * Allows the configuration of the URI Auto-Generation functionality of the module and any
 * additional exclusions to be added as required to enable third party software to work properly.
 * 
 * @package     ceon_uri_mapping
 * @author      Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @copyright   Copyright 2009-2010 Ceon
 * @copyright   Copyright 2003-2007 Zen Cart Development Team
 * @copyright   Portions Copyright 2003 osCommerce
 * @link        http://dev.ceon.net/web/zen-cart/uri_mapping
 * @license     http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version     $Id: ceon_uri_mapping_config.php 444 2010-06-16 10:05:07Z conor $
 */

/**
 * Version info - don't touch!
 */
define('CEON_URI_MAPPING_VERSION', '3.6.3');


require('includes/application_top.php');

$languages = zen_get_languages();
$num_languages = sizeof($languages);

/**
 * Load in the standard configuration variables for the module.
 */
require_once(DIR_FS_CATALOG . DIR_WS_INCLUDES . 'extra_datafiles/ceon_uri_mapping_config.php');

// Check if the database tables need be installed/updated //////////////////////////////////////////
$version = null;

$table_exists_query = 'SHOW TABLES LIKE "' . TABLE_CEON_URI_MAPPING_CONFIGS . '";';
$table_exists_result = $db->Execute($table_exists_query);

if (!$table_exists_result->EOF) {
	// Database table exists, get version info

	// Only one config currently supported so ID is hard-coded in following SQL
	$load_config_sql = "
		SELECT
			version
		FROM
			" . TABLE_CEON_URI_MAPPING_CONFIGS . "
		WHERE
			id ='1';";
	
	$load_config_result = $db->Execute($load_config_sql);
	
	if (!$load_config_result->EOF) {
		$version = $load_config_result->fields['version'];
	}
}


if (is_null($version) || $version != CEON_URI_MAPPING_VERSION) {
	@include_once('ceon_uri_mapping_auto_upgrade.php');
	
	if (!isset($ceon_uri_mapping_just_installed) &&
			(!isset($ceon_uri_mapping_upgraded) || $ceon_uri_mapping_upgraded != true)) {
		print '<strong><span style="color: red">Warning:</span><br />The Ceon URI Mapping ' .
			'Database is out of date - install/upgrade failed!</strong><br /><br />Was the ' .
			'&ldquo;ceon_uri_mapping_auto_upgrade.php&rdquo; script uploaded?';
		exit();
	} else if (isset($ceon_uri_mapping_just_installed)) {
		$messageStack->add(sprintf(SUCCESS_DATABASE_INSTALLED, CEON_URI_MAPPING_VERSION),
			'success');
		
		$version = CEON_URI_MAPPING_VERSION;
	} else {
		$messageStack->add(sprintf(SUCCESS_DATABASE_UPDATED, CEON_URI_MAPPING_VERSION),
			'success');
		
		$version = CEON_URI_MAPPING_VERSION;
	}
}


/**
 * Variables hold values for the configuration
 */
$whitespace_replacement_options = array(
	array(
		'id' => CEON_URI_MAPPING_SINGLE_UNDERSCORE,
		'text' => TEXT_WHITESPACE_REPLACEMENT_SINGLE_UNDERSCORE
		),
	array(
		'id' => CEON_URI_MAPPING_SINGLE_DASH,
		'text' => TEXT_WHITESPACE_REPLACEMENT_SINGLE_DASH
		),
	array(
		'id' => CEON_URI_MAPPING_SINGLE_FULL_STOP,
		'text' => TEXT_WHITESPACE_REPLACEMENT_SINGLE_FULL_STOP
		),
	array(
		'id' => CEON_URI_MAPPING_REMOVE,
		'text' => TEXT_WHITESPACE_REPLACEMENT_REMOVE
		)
	);



// Main variable holds list of errors (if any) for current configuration
$errors = array();

// Does a configuration need to be loaded or is one currently being edited?
if (!isset($_POST['autogen_new'])) {
	// Load the configuration
	// Only one config currently supported so ID is hard-coded in following SQL
	$load_config_sql = "
		SELECT
			autogen_new,
			whitespace_replacement,
			capitalisation,
			remove_words,
			char_str_replacements,
			excluded_files
		FROM
			" . TABLE_CEON_URI_MAPPING_CONFIGS . " 
		WHERE
			id = '1';";
	
	$load_config_result = $db->Execute($load_config_sql);
	
	if ($load_config_result->EOF) {
		
	} else {
		$autogen_new = $load_config_result->fields['autogen_new'];
		$autogen_new_yes = ($autogen_new == 1 ? true : false);
		$autogen_new_no = ($autogen_new == 0 ? true : false);
		$whitespace_replacement = $load_config_result->fields['whitespace_replacement'];
		$capitalisation = $load_config_result->fields['capitalisation'];
		$capitalisation_lowercase =
			($capitalisation == CEON_URI_MAPPING_CAPITALISATION_LOWERCASE ? true : false);
		$capitalisation_as_is =
			($capitalisation == CEON_URI_MAPPING_CAPITALISATION_AS_IS ? true : false);
		$capitalisation_ucfirst =
			($capitalisation == CEON_URI_MAPPING_CAPITALISATION_UCFIRST ? true : false);
		$remove_words = $load_config_result->fields['remove_words'];
		$char_str_replacements = $load_config_result->fields['char_str_replacements'];
		$excluded_files = $load_config_result->fields['excluded_files'];
	}
} else if (isset($_POST['autogen_new'])) {
	// Data has been submitted - check it	
	$autogen_new = $_POST['autogen_new'];
	$autogen_new_yes = ($autogen_new == 1 ? true : false);
	$autogen_new_no = ($autogen_new == 0 ? true : false);
	$whitespace_replacement = $_POST['whitespace_replacement'];
	$capitalisation = $_POST['capitalisation'];
	$capitalisation_lowercase =
		($capitalisation == CEON_URI_MAPPING_CAPITALISATION_LOWERCASE ? true : false);
	$capitalisation_as_is =
		($capitalisation == CEON_URI_MAPPING_CAPITALISATION_AS_IS ? true : false);
	$capitalisation_ucfirst =
		($capitalisation == CEON_URI_MAPPING_CAPITALISATION_UCFIRST ? true : false);
	$remove_words = trim($_POST['remove_words']);
	$char_str_replacements = trim($_POST['char_str_replacements']);
	$excluded_files = trim($_POST['excluded_files']);
	
	// Save the configuration
	if (!$ceon_uri_mapping_demo && sizeof($errors) == 0) {
		
		$save_config_data_array = array(
			'autogen_new' => $autogen_new,
			'whitespace_replacement' => $whitespace_replacement,
			'capitalisation' => $capitalisation,
			'remove_words' => ((strlen($remove_words) == 0) ? 'null' : $remove_words),
			'char_str_replacements' => ((strlen($char_str_replacements) == 0) ? 'null' : $char_str_replacements),
			'excluded_files' => zen_db_prepare_input($excluded_files)
			);
		
		// Only one config currently supported so ID is hard-coded
		$selection_sql = "id = '1'";
		
		$save_config_result = zen_db_perform(TABLE_CEON_URI_MAPPING_CONFIGS,
			$save_config_data_array, 'update', $selection_sql);
		
		if ($save_config_result) {
			if ($ceon_uri_mapping_demo) {
				$messageStack->add(SUCCESS_CONFIGURATION_SAVED_DEMO, 'success');
			} else {
				$messageStack->add(SUCCESS_CONFIGURATION_SAVED, 'success');
			}
		}
	}
}

?>
<!doctype html public "-//W3C//DTD HTML 4.01 Transitional//EN">
<html <?php echo HTML_PARAMS; ?>>
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=<?php echo CHARSET; ?>">
	<title><?php echo TITLE; ?></title>
	<link rel="stylesheet" type="text/css" href="includes/stylesheet.css">
	<link rel="stylesheet" type="text/css" href="includes/cssjsmenuhover.css" media="all" id="hoverJS">
	<script language="javascript" src="includes/menu.js"></script>
	<script language="javascript" src="includes/general.js"></script>
	<script type="text/javascript">
		<!--
		function init()
		{
			cssjsmenu('navbar');
			if (document.getElementById)
			{
				var kill = document.getElementById('hoverJS');
				kill.disabled = true;
			}
		}
		// -->
	</script>
	<style type="text/css">
	.URIMappingPageHeading { padding-top: 1em; padding-bottom: 1.2em; }
	fieldset { padding: 0.8em 0.8em; margin-bottom: 2.5em; }
	fieldset fieldset { margin-bottom: 1em; }
	legend { font-weight: bold; font-size: 1.3em; }
	
	.URIMappingConfigLabel, .URIMappingConfigField, .URIMappingConfigDesc {
		vertical-align: top;
	}
	.URIMappingConfigLabel { font-weight: bold; padding-right: 1em; }
	.URIMappingConfigLabel { width: 20%; }
	.URIMappingConfigDesc p { margin-bottom: 0.5em; }
	.URIMappingConfigField { padding-bottom: 1.3em; }
	
	.ErrorIntro { margin: 2em 0; color: #f00; }
	.FormError { font-weight: bold; color: #f00; }
	
	.Collapse { display:  none; }
	
	#footer {
		padding-top: 1em;
		text-align: right;
		font-size: 0.9em;
		padding-bottom: 2em;
	}
	#footer img {
		border: none;
	}
	#ceon_button_logo {
		float: left;
		margin-right: 14px;
	}
	#version_info {
		padding: 0;
		margin-top: 1em;
		margin-bottom: 2em;
		line-height: 1.3
	}
	#version_info a {
		font-size: 0.9em;
	}
	</style>
</head>
<body onload="init()">
<div id="spiffycalendar" class="text"></div>
<!-- header //-->
<?php require(DIR_WS_INCLUDES . 'header.php'); ?>
<!-- header_eof //-->

<!-- body //-->
<?php echo zen_draw_form('ceon_uri_mapping', FILENAME_CEON_URI_MAPPING_CONFIG, zen_get_all_get_params(), 'post', 'onsubmit="" id="ceon_uri_mapping"', true);
echo zen_hide_session_id(); ?>
<table border="0" width="100%" cellspacing="2" cellpadding="2">
	<tr>
<!-- body_text //-->
		<td width="100%" valign="top">
			<table border="0" width="100%" cellspacing="0" cellpadding="0">
				<tr>
					<td class="pageHeading URIMappingPageHeading"><?php echo HEADING_TITLE; ?></td>
				</tr>
				<tr>
					<td>
<?php
$num_errors = sizeof($errors);

if ($num_errors == 1) {
	echo '<p class="ErrorIntro">' . TEXT_ERROR_IN_CONFIG;
} else if ($num_errors > 0 ) {
	printf('<p class="ErrorIntro">' . TEXT_ERRORS_IN_CONFIG, $num_errors);
} else {
	echo '<p>';
}
?>
						</p>	
					</td>
				</tr>
				<tr>
					<td>
<fieldset>
	<legend><?php
	echo TEXT_AUTOGEN_CONFIG;
?></legend>
	<table border="0" width="100%" cellpadding="0" cellspacing="0" id="config">
		<tr>
			<td rowspan="2" class="URIMappingConfigLabel"><label for="autogen_new"><?php echo TEXT_LABEL_AUTOGEN_ENABLED_NEW; ?></label></td>
			<td class="URIMappingConfigDesc">
				<?php echo TEXT_CONFIG_DESC_AUTOGEN_ENABLED_NEW; ?>
			</td>
		</tr>
		<tr>
			<td class="URIMappingConfigField">
<?php
echo '<p>' . zen_draw_radio_field('autogen_new', '1', $autogen_new_yes) . '<label class="attribsRadioButton" for="autogen_new_yes">' . TEXT_YES . '</label>' . "\n";
echo '<br />' . zen_draw_radio_field('autogen_new', '0', $autogen_new_no) . '<label class="attribsRadioButton" for="autogen_new_no">' . TEXT_NO . '</label>' . "</p>\n";

?>
			</td>
		</tr>
		<tr>
			<td rowspan="2" class="URIMappingConfigLabel"><label for="whitespace_replacement"><?php echo TEXT_LABEL_WHITESPACE_REPLACEMENT; ?></label></td>
			<td class="URIMappingConfigDesc">
				<?php echo TEXT_CONFIG_DESC_WHITESPACE_REPLACEMENT; ?>
			</td>
		</tr>
		<tr>
			<td class="URIMappingConfigField">
<?php
echo zen_draw_pull_down_menu('whitespace_replacement', $whitespace_replacement_options, $whitespace_replacement, '') . "\n";
?>
			</td>
		</tr>
		<tr>
			<td rowspan="2" class="URIMappingConfigLabel"><label for="capitalisation"><?php echo TEXT_LABEL_CAPITALISATION; ?></label></td>
			<td class="URIMappingConfigDesc">
				<?php echo TEXT_CONFIG_DESC_CAPITALISATION; ?>
			</td>
		</tr>
		<tr>
			<td class="URIMappingConfigField">
<?php
echo '<p>' . zen_draw_radio_field('capitalisation', CEON_URI_MAPPING_CAPITALISATION_LOWERCASE, $capitalisation_lowercase) . '<label class="attribsRadioButton" for="capitalisation_lowercase">' . TEXT_CAPITALISATION_LOWERCASE . '</label>' . "\n";
echo '<br />' . zen_draw_radio_field('capitalisation', CEON_URI_MAPPING_CAPITALISATION_AS_IS, $capitalisation_as_is) . '<label class="attribsRadioButton" for="capitalisation_as_is">' . TEXT_CAPITALISATION_AS_IS . '</label>' . "\n";
echo '<br />' . zen_draw_radio_field('capitalisation', CEON_URI_MAPPING_CAPITALISATION_UCFIRST, $capitalisation_ucfirst) . '<label class="attribsRadioButton" for="capitalisation_ucfirst">' . TEXT_CAPITALISATION_UCFIRST . '</label>' . "</p>\n";
?>
			</td>
		</tr>
		<tr>
			<td rowspan="2" class="URIMappingConfigLabel"><label for="remove_words"><?php echo TEXT_LABEL_REMOVE_WORDS; ?></label></td>
			<td class="URIMappingConfigDesc">
				<?php echo TEXT_CONFIG_DESC_REMOVE_WORDS; ?>
			</td>
		</tr>
		<tr>
			<td class="URIMappingConfigField">
<?php
echo zen_draw_textarea_field('remove_words', 'virtual', 50, 6, $remove_words) . "\n";
?>
			</td>
		</tr>
		<tr>
			<td rowspan="2" class="URIMappingConfigLabel"><label for="char_str_replacements"><?php echo TEXT_LABEL_CHAR_STR_REPLACEMENTS; ?></label></td>
			<td class="URIMappingConfigDesc">
				<?php echo TEXT_CONFIG_DESC_CHAR_STR_REPLACEMENTS; ?>
			</td>
		</tr>
		<tr>
			<td class="URIMappingConfigField">
<?php
echo zen_draw_textarea_field('char_str_replacements', 'virtual', 50, 2, $char_str_replacements) . "\n";
?>
			</td>
		</tr>
	</table>
</fieldset>

<fieldset>
	<legend><?php
	echo TEXT_EXCLUDED_FILES;
?></legend>
	<table border="0" width="100%" cellpadding="0" cellspacing="0" id="config">
		<tr>
			<td rowspan="2" class="URIMappingConfigLabel"><label for="excluded_files"><?php echo TEXT_LABEL_EXCLUDED_FILES; ?></label></td>
			<td class="URIMappingConfigDesc">
				<?php
				$excluded_files_list = '';
				$num_excluded_files = sizeof($ceon_uri_mapping_excluded_files);
				
				for ($i = 0; $i < $num_excluded_files; $i++) {
					$excluded_files_list .= strtolower(DIR_WS_CATALOG) .
						$ceon_uri_mapping_excluded_files[$i] . ', ';
				}
				
				$excluded_files_list = substr($excluded_files_list, 0,
					strlen($excluded_files_list) - 2);
				
				echo str_replace('{standard_excluded_files}', $excluded_files_list,
					TEXT_CONFIG_DESC_EXCLUDED_FILES);
				
				?>
			</td>
		</tr>
		<tr>
			<td class="URIMappingConfigField">
<?php
echo zen_draw_textarea_field('excluded_files', 'virtual', 50, 6, $excluded_files) . "\n";
?>
			</td>
		</tr>
	</table>
</fieldset>
					</td>
				</tr>
				<tr>
					<td align="right">
						<?php
						echo zen_image_submit('button_save.gif', IMAGE_SAVE, 'name="save" value="save"');
						echo '&nbsp;<a href="' . zen_href_link(FILENAME_DEFAULT, '', 'NONSSL') .'">' . zen_image_button('button_cancel.gif', IMAGE_CANCEL) . '</a>';
						?>
					</td>
				</tr>
				<tr>
					<td style="padding-top: 3em;"><?php echo zen_draw_separator('pixel_black.gif', '100%', '1'); ?></td>
				</tr>
				<tr>
					<td id="footer"><a href="http://dev.ceon.net/web" target="_blank"><img src="<?php echo DIR_WS_ADMIN . DIR_WS_IMAGES; ?>ceon_button_logo.png" alt="Ceon" id="ceon_button_logo" /></a>Module &copy; Copyright 2008-2010 <a href="http://dev.ceon.net/web" target="_blank">Ceon</a>
					<p id="version_info">Module Version: <?php echo $version; ?><br />
					<a href="http://dev.ceon.net/web/zen-cart/uri_mapping/version_checker/<?php echo $version; ?>" target="_blank">Check for Updates</a></p>
				</tr>
			</table>
		</td>
<!-- body_text_eof //-->
	</tr>
</table>
</form>
<!-- body_eof //-->

<!-- footer //-->
<?php require(DIR_WS_INCLUDES . 'footer.php'); ?>
<!-- footer_eof //-->
</body>
</html>
<?php require(DIR_WS_INCLUDES . 'application_bottom.php'); ?>