<?php

/**
 * Ceon URI Mapping Functions.
 *
 * @author      Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @copyright   Copyright 2006-2010 Ceon
 * @copyright   Copyright 2004-2006 Aidan Lister
 * @link        http://dev.ceon.net/web/zen-cart/sage_pay_direct
 * @license     http://www.gnu.org/copyleft/gpl.html   GNU Public License V2.0
 * @version     $Id: ceon_uri_mapping_functions.php 444 2010-06-16 10:05:07Z conor $
 */


// {{{ ceon_uri_mapping_buildQueryString()

/**
 * Builds the query string for the current query parameters. Can handle arrays and multi-dimensional
 * arrays!
 *
 * @author      Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @link        http://dev.ceon.net/web/zen-cart/uri_mapping
 * @return      string   A URL encoded query parameters string.
 */
function ceon_uri_mapping_buildQueryString()
{
	$query_string = '';
	
	foreach ($_GET as $key => $value) {
		if (is_array($value)) {			
			$query_string .= ceon_uri_mapping_buildArrayQueryParameter($key, $value);
		} else {
			$query_string .= '&' . urlencode($key) . '=' . urlencode($value);
		}
	}
	
	if ($query_string != '') {
		$query_string = '?' . substr($query_string, 1, strlen($query_string) - 1);
	}
	
	return $query_string;
}

// }}}


// {{{ ceon_uri_mapping_buildArrayQueryParameter()

/**
 * Builds the string for an array query parameter.
 *
 * @author      Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @link        http://dev.ceon.net/web/zen-cart/uri_mapping
 * @param       string   $key     The key for the query string parameter.
 * @param       string   $value   The value for the query string parameter.
 * @return      string   A URL encoded string representation of the query parameter.
 */
function ceon_uri_mapping_buildArrayQueryParameter($key, $value)
{
	$parameter_string = '';
	
	$key .= '[]';
	
	foreach ($value as $subvalue) {
		if (is_array($subvalue)) {
			$parameter_string .= ceon_uri_mapping_buildArrayQueryParameter($key, $subvalue);
		} else {
			$parameter_string .= '&' . urlencode($key) . '=' . urlencode($subvalue);
		}
	}
	
	return $parameter_string;
}

// }}}

?>