<?php

/**
 * Ceon URI Mapping URI Processing Script.
 * 
 * @package     ceon_uri_mapping
 * @author      Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @copyright   Copyright 2009-2010 Ceon
 * @copyright   Copyright 2003-2007 Zen Cart Development Team
 * @copyright   Portions Copyright 2003 osCommerce
 * @link        http://dev.ceon.net/web/zen-cart/uri_mapping
 * @license     http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version     $Id: init_ceon_uri_mapping.php 444 2010-06-16 10:05:07Z conor $
 */

if (!defined('IS_ADMIN_FLAG')) {
	die('Illegal Access');
}

// Make sure rewrite rules haven't interfered with value of server variable used by some Zen Cart
// functionality
if ($PHP_SELF == '/') {
	$PHP_SELF = '/index.php';
} else if ($PHP_SELF == DIR_WS_CATALOG) {
	$PHP_SELF = DIR_WS_CATALOG . 'index.php';
}

if (defined('CEON_URI_MAPPING_ENABLED') && CEON_URI_MAPPING_ENABLED == 1) {
	// Check if IIS ISAPI rewrite module is being used and, if so, use the original URI it provides
	if (isset($_SERVER['HTTP_X_REWRITE_URL'])) {
		$_SERVER['REQUEST_URI'] = $_SERVER['HTTP_X_REWRITE_URL'];
	}
	
	if (!isset($_SERVER['REQUEST_URI'])) {
		print 'Ceon URI Mapping cannot access REQUEST_URI server variable!';
	} else {
		$request_uri = $_SERVER['REQUEST_URI'];
		
		// Remove and store any query string
		$query_string = null;
		
		$query_pos = strpos($request_uri, '?');
		
		if ($query_pos !== false) {
			$query_string = substr($request_uri, $query_pos + 1,
				strlen($request_uri) - $query_pos - 1);
			
			$request_uri = substr($request_uri, 0, $query_pos);
			
			if (isset($_GET[zen_session_name()])) {
				// Remove any session ID from the query string as it certainly won't be matched
				// against in the database!
				$session_parameter_pattern = '/&?' . zen_session_name() . '=[^&]+/i';
				
				$query_string = preg_replace($session_parameter_pattern, '', $query_string);
			}
		}
		
		// Don't process any excluded files
		// First prepare paths of default exclusions for comparison against the request URI
		$num_excluded_files = sizeof($ceon_uri_mapping_excluded_files);
		
		for ($i = 0; $i < $num_excluded_files; $i++) {
			$ceon_uri_mapping_excluded_files[$i] = strtolower(DIR_WS_CATALOG) .
				$ceon_uri_mapping_excluded_files[$i];
		}
		
		// Now load and prepare the paths of any specific exclusions for the store
		$get_excluded_files_query = "
			SELECT
				excluded_files
			FROM
				" . TABLE_CEON_URI_MAPPING_CONFIGS . "
			WHERE
				id = '1';";
		
		$get_excluded_files_result = $db->Execute($get_excluded_files_query);
		
		if (!$get_excluded_files_result->EOF) {
			$excluded_files_string = $get_excluded_files_result->fields['excluded_files'];
			
			$excluded_files_array = explode(',', $excluded_files_string);
			
			foreach ($excluded_files_array as $excluded_file) {
				$ceon_uri_mapping_excluded_files[] = strtolower(trim($excluded_file));
			}
		}
		
		if (!in_array(strtolower($request_uri), $ceon_uri_mapping_excluded_files)) {
			// Run the language initsystem code here as, for some strange reason, it runs late in
			// the initsystem process and the Ceon URI Mapping module must run before the sanitize
			// initsystem script which normally preceeds the language initsystem code.
			
			// Record if the language is currently being changed by the user
			$language_changed = false;
			$original_language_id = (isset($_SESSION['languages_id']) ? $_SESSION['languages_id'] :
				null);
			
			if (!isset($_SESSION['language']) || isset($_GET['language'])) {
				$lng = new language();
				
				if (isset($_GET['language']) && zen_not_null($_GET['language'])) {
					$lng->set_language($_GET['language']);
					
					$language_changed = true;
					unset($_GET['language']);
				} else {
					if (LANGUAGE_DEFAULT_SELECTOR == 'Browser') {
						$lng->get_browser_language();
					} else {
						$lng->set_language(DEFAULT_LANGUAGE);
					}
				}
				
				$_SESSION['language'] = (zen_not_null($lng->language['directory']) ?
					$lng->language['directory'] : 'english');
				
				$_SESSION['languages_id'] = (zen_not_null($lng->language['id']) ?
					$lng->language['id'] : 1);
				
				$_SESSION['languages_code'] = (zen_not_null($lng->language['code']) ?
					$lng->language['code'] : 'en');
			}
			
			if ($language_changed && $original_language_id == $_SESSION['languages_id']) {
				$language_changed = false;
			}
			
			if (is_null($original_language_id)) {
				$original_language_id = $_SESSION['languages_id'];
			}
			
			// Handle variations of index page URI
			if ((isset($_GET['main_page']) && $_GET['main_page'] == FILENAME_DEFAULT &&
					!isset($_GET['cPath']) && !isset($_GET['manufacturers_id']) &&
					!isset($_GET['typefilter'])) ||
					(!isset($_GET['main_page']) &&
					strtolower($request_uri) == strtolower(DIR_WS_CATALOG) . 'index.php')) {
				// URI is DIR_WS_CATALOG/index.php?main_page=index or DIR_WS_CATALOG/index.php
				// Redirect to DIR_WS_CATALOG
				$redirection_uri = DIR_WS_CATALOG;
				
				if (isset($_GET['main_page'])) {
					unset($_GET['main_page']);
				}
				
				require_once(DIR_FS_CATALOG . DIR_WS_FUNCTIONS . 'ceon_uri_mapping_functions.php');
				
				$query_string = ceon_uri_mapping_buildQueryString();
				
				$redirection_uri .= $query_string;
				
				header("HTTP/1.1 301 Moved Permanently");
				header("Location: " . $redirection_uri);
				zen_exit();
			} else if (!isset($_GET['main_page']) &&
					strtolower($request_uri) == strtolower(DIR_WS_CATALOG) ||
					(strtolower($request_uri) . '/') == strtolower(DIR_WS_CATALOG)) {
				$_GET['main_page'] = FILENAME_DEFAULT;
				
				if (!isset($_GET['cPath']) && !isset($_GET['manufacturers_id']) &&
						!isset($_GET['typefilter'])) {
					// Make this URI the canonical URI for the page!
					$ceon_uri_mapping_canonical_uri = HTTP_SERVER . DIR_WS_CATALOG;
					
					// Remove any trailing slash from the canonical URI, unless the URI is the root
					// of the site
					if (strlen(DIR_WS_CATALOG) > 1 && substr(DIR_WS_CATALOG, -1) == '/') {
						$ceon_uri_mapping_canonical_uri = substr($ceon_uri_mapping_canonical_uri, 0,
							strlen($ceon_uri_mapping_canonical_uri) - 1);
					}
				}
			}
			
			if (!isset($_GET['main_page'])) {
				// Standard Zen Cart URI isn't being used, attempt to identify and map URI.
				
				// Attempt to match slight variations of the URI by matching against a regular
				// expression that allows for an optional trailing slash
				// First make sure that request URI has no invalid characters in it
				$uri_to_match = preg_replace('/[^a-zA-Z0-9_\-\.\/%]/', '', $request_uri);
				
				if (substr($uri_to_match, -1) == '/') {
					// Remove the trailing slash to prevent duplication in the regular expression
					$uri_to_match = substr($uri_to_match, 0, strlen($uri_to_match) -1);
				}
				
				$pattern = '^' . $uri_to_match . '\/?$';
				
				// LIKE operator reintroduced to filter matches to smaller number of possible rows
				// before using the REGEXP operator to specifically match the URI. Code suggested
				// by Christian Pinder
				// @TODO Rework the database to enable the use of indexes and a simple LIKE (no
				// wildcard)
				$identify_main_page_query = "
					SELECT
						um.language_id,
						um.main_page,
						um.query_string_parameters,
						um.associated_db_id,
						um.alternate_uri,
						um.current_uri,
						um.redirection_type_code
					FROM
						" . TABLE_CEON_URI_MAPPINGS . " um
					WHERE
						um.uri LIKE '" . zen_db_prepare_input($uri_to_match) . "%' AND
						um.uri REGEXP '" . zen_db_prepare_input($pattern) . "'
					ORDER BY
						um.current_uri DESC,
						um.language_id,
						um.date_added DESC;";
				
				$identify_main_page_result = $db->Execute($identify_main_page_query);
				
				if (!$identify_main_page_result->EOF) {
					// Have matched the URI!
					
					// Make this URI the canonical URI for the page!
					$ceon_uri_mapping_canonical_uri = HTTP_SERVER . $uri_to_match;
					
					// Use the current mapping which specifically maps the language ID originally
					// being used when this URI was accessed, otherwise simply use the most recent
					// current mapping, falling back to the most recent mapping for the original
					// language ID and as a last resort the most recent mapping for any ID
					$current_and_original_language_mapping = null;
					$current_mappings = array();
					$original_language_mappings = array();
					$other_mappings = array();
					
					while (!$identify_main_page_result->EOF) {
						$current_mapping_info = $identify_main_page_result->fields;
						
						if ($current_mapping_info['current_uri'] == 1 &&
								$current_mapping_info['language_id'] == $original_language_id) {
							// Optimal mapping found
							$current_and_original_language_mapping = $current_mapping_info;
							break;
						}
						
						if ($current_mapping_info['current_uri'] == 1) {
							$current_mappings[] = $current_mapping_info;
						} else if ($current_mapping_info['language_id'] == $original_language_id) {
							$original_language_mappings[] = $current_mapping_info;
						} else {
							$other_mappings[] = $current_mapping_info;
						}
						
						$identify_main_page_result->MoveNext();
					}
					
					// Use the details for the best possible match for the URI (criteria as above)
					if (!is_null($current_and_original_language_mapping)) {
						$mapping_info = $current_and_original_language_mapping;
					} else if (sizeof($current_mappings) > 0) {
						$mapping_info = $current_mappings[0];
					} else if (sizeof($original_language_mappings) > 0) {
						$mapping_info = $original_language_mappings[0];
					} else {
						$mapping_info = $other_mappings[0];
					}
					
					$language_id = $mapping_info['language_id'];
					$main_page = $mapping_info['main_page'];
					$query_string_parameters = $mapping_info['query_string_parameters'];
					$associated_db_id = $mapping_info['associated_db_id'];
					$alternate_uri = $mapping_info['alternate_uri'];
					$current_uri = $mapping_info['current_uri'];
					$redirection_type_code = $mapping_info['redirection_type_code'];
					
					if (!is_null($alternate_uri)) {
						$alternate_uri = trim($alternate_uri);
						if (strlen($alternate_uri) < 2) {
							$alternate_uri = null;
						}
					}
					
					if (!is_null($main_page) && is_null($alternate_uri)) {
						// Have matched a Zen Cart page to initialise
						
						// Is the URI a current URI or should an attempt be made to find the current
						// URI to redirect to? Or has the user just changed the language so an
						// attempt should be made to find the current URI for the new language?
						if ($current_uri != 1 || $language_changed) {
							// Attempt to find the current URI for the user's chosen language
							if (!is_null($associated_db_id)) {
								$redirection_uri_query = "
									SELECT
										um.uri
									FROM
										" . TABLE_CEON_URI_MAPPINGS . " um
									WHERE
										um.main_page = '" . zen_db_input(zen_db_prepare_input(
											$main_page)) . "'
									AND
										um.associated_db_id = '" . (int) $associated_db_id . "'
									AND
										um.language_id = '" . (int) $_SESSION['languages_id'] . "'
									AND
										um.current_uri = '1';";
							} else if (!is_null($query_string_parameters)) {
								$redirection_uri_query = "
									SELECT
										um.uri
									FROM
										" . TABLE_CEON_URI_MAPPINGS . " um
									WHERE
										um.main_page = '" . zen_db_input(zen_db_prepare_input(
											$main_page)) . "'
									AND
										um.query_string_parameters = '" . zen_db_input(
											zen_db_prepare_input($query_string)) . "'
									AND
										um.language_id = '" . (int) $_SESSION['languages_id'] . "'
									AND
										um.current_uri = '1';";
							} else {
								$redirection_uri_query = "
									SELECT
										um.uri
									FROM
										" . TABLE_CEON_URI_MAPPINGS . " um
									WHERE
										um.main_page = '" . zen_db_input(zen_db_prepare_input(
											$main_page)) . "'
									AND
										um.associated_db_id IS NULL
									AND
										um.query_string_parameters IS NULL
									AND
										um.language_id = '" . (int) $_SESSION['languages_id'] . "'
									AND
										um.current_uri = '1';";
							}
							
							$redirection_uri_result = $db->Execute($redirection_uri_query);
							
							if ($redirection_uri_result->EOF) {
								// Didn't find a current URI mapping for the user's chosen language
								
								// If the URI mapping identified so far is a historical mapping
								// which uses a different language, try to find the current URI
								// mapping for that language
								if ($current_uri != 1 &&
										$language_id != $_SESSION['languages_id']) {
									if (!is_null($associated_db_id)) {
										$redirection_uri_query = "
											SELECT
												um.uri
											FROM
												" . TABLE_CEON_URI_MAPPINGS . " um
											WHERE
												um.main_page = '" . zen_db_input(
													zen_db_prepare_input($main_page)) . "'
											AND
												um.associated_db_id = '" .
													(int) $associated_db_id . "'
											AND
												um.language_id = '" . (int) $language_id . "'
											AND
												um.current_uri = '1';";
									} else if (!is_null($query_string_parameters)) {
										$redirection_uri_query = "
											SELECT
												um.uri
											FROM
												" . TABLE_CEON_URI_MAPPINGS . " um
											WHERE
												um.main_page = '" . zen_db_input(
													zen_db_prepare_input($main_page)) . "'
											AND
												um.query_string_parameters = '" . zen_db_input(
													zen_db_prepare_input($query_string)) . "'
											AND
												um.language_id = '" . (int) $language_id . "'
											AND
												um.current_uri = '1';";
									} else {
										$redirection_uri_query = "
											SELECT
												um.uri
											FROM
												" . TABLE_CEON_URI_MAPPINGS . " um
											WHERE
												um.main_page = '" . zen_db_input(
													zen_db_prepare_input($main_page)) . "'
											AND
												um.associated_db_id IS NULL
											AND
												um.query_string_parameters IS NULL
											AND
												um.language_id = '" . (int) $language_id . "'
											AND
												um.current_uri = '1';";
									}
									
									$redirection_uri_result = $db->Execute($redirection_uri_query);
									
									if ($redirection_uri_result->EOF) {
										// Didn't find a current URI mapping for the language the
										// identified mapping is using
										
									} else {
										// If using a different language from the session (and the
										// user hasn't just selected a new language), must use the
										// new language
										if ($language_id != $_SESSION['languages_id'] &&
												!$language_changed) {
											$language_info_query = "
												SELECT
													directory,
													code
												FROM
													" . TABLE_LANGUAGES . " 
												WHERE
													languages_id = '" . (int) $language_id . "';";
											
											$language_info_result =
												$db->Execute($language_info_query);
											
											$_SESSION['language'] =
												$language_info_result->fields['directory'];
											$_SESSION['languages_id'] = $language_id;
											$_SESSION['languages_code'] =
												$language_info_result->fields['code'];
										}
									}
								}
							}
							
							if (!$redirection_uri_result->EOF) {
								// Have found a current URI mapping for this page, force a redirect
								// to it
								$redirection_uri = $redirection_uri_result->fields['uri'];
								
								// Include any extra info in the GET variables
								if (!is_null($associated_db_id) ||
										is_null($query_string_parameters)) {
									require_once(DIR_FS_CATALOG . DIR_WS_FUNCTIONS .
										'ceon_uri_mapping_functions.php');
									
									$query_string = ceon_uri_mapping_buildQueryString();
								} else if (!is_null($query_string_parameters)) {
									// Query string variables will be rebuilt after redirecting so
									// no need to append them
									$query_string = '';
								}
								
								$redirection_uri .= $query_string;
								
								header("HTTP/1.1 301 Moved Permanently");
								header("Location: " . $redirection_uri);
								zen_exit();
							} else if (!$language_changed) {
								// There is no current URI for this language, this is a URI from the
								// historical database being used to prevent broken links. Force a
								// redirect to standard Zen Cart page
								
								if (!is_null($associated_db_id)) {
									// Must supply the ID information necessary for the correct info
									// to be loaded
									switch ($main_page) {
										case FILENAME_DEFAULT:
											if (!isset($_GET['cPath'])) {
												$_GET['cPath'] = $associated_db_id;
											}
											break;
										case FILENAME_EZPAGES:
										case FILENAME_EZPAGES_POPUP:
											$_GET['id'] = $associated_db_id;
											break;
										default:
											// This is a product related page
											$_GET['products_id'] = $associated_db_id;
											break;
									}
								} else if (!is_null($query_string_parameters)) {
									// Must make sure that all parameters for this URI are loaded
									$query_string_pairs = explode('&', $query_string_parameters);
									
									foreach ($query_string_pairs as $query_string_pair) {
										$parameter_parts = explode('=', $query_string_pair);
										
										// Parameter from database overrides any in query string as
										// this is a historical URI mapping.
										if (sizeof($parameter_parts) == 2) {
											$_GET[$parameter_parts[0]] =
												urldecode($parameter_parts[1]);
										}
									}
								}
								
								if ($request_type == 'SSL') {
									$redirection_uri = DIR_WS_HTTPS_CATALOG;
								} else {
									$redirection_uri = DIR_WS_CATALOG;
								}
								
								if (substr($redirection_uri, 0, 1) != '/') {
									$redirection_uri = '/' . $redirection_uri;
								}
								
								$redirection_uri .= 'index.php';
								
								// Add the page to be loaded to the list of query parameters
								$_GET['main_page'] = $main_page;
								
								require_once(DIR_FS_CATALOG . DIR_WS_FUNCTIONS .
									'ceon_uri_mapping_functions.php');
								
								$query_string = ceon_uri_mapping_buildQueryString();
								
								$redirection_uri .= $query_string;
								
								// Record that the source of redirect was this script, avoiding any
								// unnecessary processing later
								$_SESSION['ceon_uri_mapping_redirected'] = true;
								
								header("HTTP/1.1 301 Moved Permanently");
								header("Location: " . $redirection_uri);
								zen_exit();
							}
						} else if ($language_id != $_SESSION['languages_id']) {
							// The user hasn't just selected a new language, but the URI uses a
							// different language from the session, so must switch the session to
							// use the new language
							$language_info_query = "
								SELECT
									directory,
									code
								FROM
									" . TABLE_LANGUAGES . " 
								WHERE
									languages_id = '" . (int) $language_id . "';";
							
							$language_info_result = $db->Execute($language_info_query);
							
							$_SESSION['language'] = $language_info_result->fields['directory'];
							$_SESSION['languages_id'] = $language_id;
							$_SESSION['languages_code'] = $language_info_result->fields['code'];
						}
						
						// This URI is the current URI for this page
						$_GET['main_page'] = $main_page;
						
						if (!is_null($associated_db_id)) {
							// Page is a category, EZ-Page or a product info page, must initialise
							// the appropriate ID
							if ($main_page == FILENAME_DEFAULT) {
								// This is a category page
								// Must build the full contextual link to the category, so any
								// hierarchy is respected
								$categories = array();
								zen_get_parent_categories($categories, $associated_db_id);
								
								$categories = array_reverse($categories);
								
								$cPath = implode('_', $categories);
								
								if (zen_not_null($cPath)) {
									$cPath .= '_';
								}
								$cPath .= $associated_db_id;
								
								$_GET['cPath'] = $cPath;
							} else if ($main_page == FILENAME_EZPAGES ||
									$main_page == FILENAME_EZPAGES_POPUP) {
								// Have found an EZ-Page. Should it be displayed as a normal page or
								// in a popup?
								if ($main_page == FILENAME_EZPAGES_POPUP ||
										isset($_GET['ezpopup'])) {
									$_GET['main_page'] = FILENAME_EZPAGES_POPUP;
								} else {
									$_GET['main_page'] = FILENAME_EZPAGES;
								}
								$_GET['id'] = $associated_db_id;
							} else {
								// This is a product related page
								if (isset($_GET['products_id']) &&
										strpos($_GET['products_id'], ':') !== false) {
									// Query string parameter includes information about selected
									// attributes (which is added when linking from shopping cart
									// to product info page) so don't override the parameter's value
									
								} else {
									$_GET['products_id'] = $associated_db_id;
								}
								
								// Rebuild the cPath variable for this page if it doesn't exist
								if (!isset($_GET['cPath'])) {
									$_GET['cPath'] = zen_get_product_path($_GET['products_id']);
								}
							}
						} else if (!is_null($query_string_parameters)) {
							// Page is a Zen Cart page with preset parameters, must initialise the
							// parameters
							$query_string_pairs = explode('&', $query_string_parameters);
							
							foreach ($query_string_pairs as $query_string_pair) {
								$parameter_parts = explode('=', $query_string_pair);
								
								// Parameter from database overrides any in query string as this is
								// a historical URI mapping.
								if (sizeof($parameter_parts) == 2) {
									$_GET[$parameter_parts[0]] = urldecode($parameter_parts[1]);
								}
							}
						}
					} else if (!is_null($alternate_uri)) {
						// Alternative URI is a full URI to be redirected to
						switch ($redirection_type_code) {
							case 301:
								header("HTTP/1.1 301 Moved Permanently");
								break;
							case 302:
								header("HTTP/1.1 302 Found");
								break;
							case 303:
								header("HTTP/1.1 302 See Other ");
								break;
							case 307:
								header("HTTP/1.1 302 Temporary Redirect");
								break;
						}
						
						header("Location: " . $alternate_uri);
						zen_exit();
					} else {
						header('HTTP/1.1 404 Not Found');
						$_GET['main_page'] = 'page_not_found';
						
						print "Unable to map identified URI: $request_uri";
					}
				} else {
					if (MISSING_PAGE_CHECK == 'On' || MISSING_PAGE_CHECK == 'true') {
						$_GET['main_page'] = 'index';
					} elseif (MISSING_PAGE_CHECK == 'Page Not Found') {
						header('HTTP/1.1 404 Not Found');
						$_GET['main_page'] = 'page_not_found';
					}
				}
			} else if (isset($_GET['main_page']) &&
					isset($_SESSION['ceon_uri_mapping_redirected'])) {
				// This script redirected the customer here so there's no need for any further
				// checks
				unset($_SESSION['ceon_uri_mapping_redirected']);
			} else if (isset($_GET['main_page']) && !isset($_GET['action']) &&
					!($_GET['main_page'] == FILENAME_DEFAULT &&
					!isset($_GET['cPath']) && !isset($_GET['manufacturers_id']) &&
					(!isset($_GET['typefilter']) || $_GET['typefilter'] == '') &&
					(isset($_GET['typefilter']) && (!isset($_GET[$_GET['typefilter'] . '_id']) ||
					$_GET[$_GET['typefilter'] . '_id'] == '')))) {
				// This is a standard Zen Cart dynamic URI - redirect it to its mapped URI if there
				// is one (but not if a form POST is taking place, as indicated by the action
				// parameter being present)
				$redirection_uri = null;
				
				$associated_db_id = null;
				$query_string_to_match = null;
				
				if ($_GET['main_page'] == FILENAME_DEFAULT && (isset($_GET['cPath']) ||
						isset($_GET['manufacturers_id']) || (isset($_GET['typefilter']) &&
						$_GET['typefilter'] != '' &&
						isset($_GET[$_GET['typefilter'] . '_id']) &&
						$_GET[$_GET['typefilter'] . '_id'] != ''))) {
					if (isset($_GET['manufacturers_id'])) {
						// This is a manufacturer page - get the URI for the manufacturers page if
						// there is one
						$query_string_to_match = 'manufacturers_id=' . $_GET['manufacturers_id'];
					} else if (isset($_GET['typefilter']) && $_GET['typefilter'] != '' &&
							isset($_GET[$_GET['typefilter'] . '_id']) &&
							$_GET[$_GET['typefilter'] . '_id'] != '') {
						// This is a filtered page - get the URI for the filtered page if there is
						// one
						$query_string_to_match = 'typefilter=' . $_GET['typefilter'] . '&' .
							$_GET['typefilter'] . '_id=' . $_GET[$_GET['typefilter'] . '_id'];
					} else if (isset($_GET['cPath'])) {
						// This is a category page - get the URI for the category ID
						$category_parts = explode('_', $_GET['cPath']);
						$category_id = $category_parts[sizeof($category_parts) - 1];
						
						$associated_db_id = $category_id;
					}
				} else if($_GET['main_page'] == FILENAME_DOCUMENT_GENERAL_INFO ||
						$_GET['main_page'] == FILENAME_DOCUMENT_PRODUCT_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_BOOK_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_FREE_SHIPPING_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_MUSIC_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS ||
						$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_WRITE ||
						$_GET['main_page'] == FILENAME_TELL_A_FRIEND) {
					$associated_db_id = $_GET['products_id'];
				} else if ($_GET['main_page'] == FILENAME_EZPAGES ||
						$_GET['main_page'] == FILENAME_EZPAGES_POPUP) {
					$associated_db_id = $_GET['id'];
				} else if ((sizeof($_GET) > 1 && !isset($_GET[zen_session_name()])) ||
						(isset($_GET[zen_session_name()]) && sizeof($_GET) > 2)) {
					// Check if there's a query string to match against for this Zen Cart page
					// Get the query string
					$query_pos = strpos($_SERVER['REQUEST_URI'], '?');
					if ($query_pos !== false) {
						$query_string_to_match = substr($_SERVER['REQUEST_URI'], $query_pos + 1,
							strlen($_SERVER['REQUEST_URI']) - $query_pos - 1);
						
						// Remove main_page variable from query string
						$query_string_to_match = str_replace('main_page=' . $_GET['main_page'], '',
							$query_string_to_match);
						
						if (substr($query_string_to_match, 0, 1) == '&') {
							$query_string_to_match = substr($query_string_to_match, 1,
								strlen($query_string_to_match) - 1);
						}
						if (substr($query_string_to_match, -1) == '&') {
							$query_string_to_match = substr($query_string_to_match, 0,
								strlen($query_string_to_match) - 1);
						}
						
						if (isset($_GET[zen_session_name()])) {
							// Remove any session ID from the query string as it certainly won't be
							// matched against in the database!
							$session_parameter_pattern = '/&?' . zen_session_name() . '=[^&]+/i';
							$query_string_to_match = preg_replace($session_parameter_pattern, '',
								$query_string_to_match);
						}
					}
				}
				
				if (!is_null($associated_db_id)) {
					$redirection_uri_query = "
						SELECT
							um.uri
						FROM
							" . TABLE_CEON_URI_MAPPINGS . " um
						WHERE
							um.main_page = '" . zen_db_input(zen_db_prepare_input(
								$_GET['main_page'])) . "'
						AND
							um.associated_db_id = '" . (int) $associated_db_id . "'
						AND
							um.language_id = '" . (int) $_SESSION['languages_id'] . "'
						AND
							um.current_uri = '1';";
				} else if (!is_null($query_string_to_match)) {
					$redirection_uri_query = "
						SELECT
							um.uri
						FROM
							" . TABLE_CEON_URI_MAPPINGS . " um
						WHERE
							um.main_page = '" . zen_db_input(zen_db_prepare_input(
								$_GET['main_page'])) . "'
						AND
							um.query_string_parameters = '" . zen_db_input(zen_db_prepare_input(
								$query_string_to_match)) . "'
						AND
							um.language_id = '" . (int) $_SESSION['languages_id'] . "'
						AND
							um.current_uri = '1';";
				} else {
					$redirection_uri_query = "
						SELECT
							um.uri
						FROM
							" . TABLE_CEON_URI_MAPPINGS . " um
						WHERE
							um.main_page = '" . zen_db_input(zen_db_prepare_input(
								$_GET['main_page'])) . "'
						AND
							um.associated_db_id IS NULL
						AND
							um.query_string_parameters IS NULL
						AND
							um.language_id = '" . (int) $_SESSION['languages_id'] . "'
						AND
							um.current_uri = '1';";
				}
				
				$redirection_uri_result = $db->Execute($redirection_uri_query);
				
				if (!$redirection_uri_result->EOF) {
					// Have found a URI mapping for this page, use this as the "current" URI for
					// this page by forcing a redirect to it
					$redirection_uri = $redirection_uri_result->fields['uri'];
					
					// Perform a sanity check to make sure that the URI to be redirected to isn't
					// simply the current URI 
					if ($request_uri != $redirection_uri) {
						// Fine to redirect
						
						// Remove unnecessary variables from query string
						if ($_GET['main_page'] == FILENAME_DEFAULT &&
								isset($_GET['manufacturer_id'])) {
							// Remove the manufacturer ID from the URI as it will be regenerated
							// when the static URI is loaded
							unset($_GET['manufacturer_id']);
						} else if ($_GET['main_page'] == FILENAME_DEFAULT &&
								isset($_GET['typefilter']) && $_GET['typefilter'] != '' &&
								isset($_GET[$_GET['typefilter'] . '_id']) &&
								$_GET[$_GET['typefilter'] . '_id'] != '') {
							// Remove the filter info from the URI as it will be regenerated when
							// the static URI is loaded
							unset($_GET['typefilter']);
							unset($_GET[$_GET['typefilter'] . '_id']);
						} else if ($_GET['main_page'] == FILENAME_DEFAULT && isset($_GET['cPath'])) {
							// Remove the category ID from the URI as it will be regenerated when
							// the static URI is loaded
							unset($_GET['cPath']);
						} else if($_GET['main_page'] == FILENAME_DOCUMENT_GENERAL_INFO ||
								$_GET['main_page'] == FILENAME_DOCUMENT_PRODUCT_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_BOOK_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_FREE_SHIPPING_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_MUSIC_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS ||
								$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_WRITE ||
								$_GET['main_page'] == FILENAME_TELL_A_FRIEND) {
							// Remove the category ID from the URI if it is simply the product's
							// master category as it will be regenerated when the static URI is
							// loaded
							if (isset($_GET['cPath']) && $_GET['cPath'] ==
									zen_get_product_path($_GET['products_id'])) {
								unset($_GET['cPath']);
							}
							
							unset($_GET['products_id']);
						} else if ($_GET['main_page'] == FILENAME_EZPAGES ||
								$_GET['main_page'] == FILENAME_EZPAGES_POPUP) {
							unset($_GET['id']);
						}
						
						unset($_GET['main_page']);
						
						if (!is_null($query_string_to_match)) {
							// Remove the query string parameters which matched those in the string
							// in the database as they will be set automatically using the same
							// parameter string associated with the mapped URI in the database -
							// this means they don't have to be displayed to the customer!
							// All other parameters should be passed or additional functionality on
							// the mapped page would not work correctly!
							$query_string_parameter_pairs = explode('&', $query_string_to_match);
							
							foreach ($query_string_parameter_pairs as
									$current_query_string_parameter_pair) {
								$key_and_value = explode('=', $current_query_string_parameter_pair);
								
								unset($_GET[$key_and_value[0]]);
							}
						}
					} else {
						// Don't redirect to the same address!
						$redirection_uri = null;
					}
				} else if (is_null($associated_db_id) && !is_null($query_string_to_match) &&
						$_GET['main_page'] != FILENAME_DEFAULT) {
					// Could this possibly be a Zen Cart page without parameters?
					$redirection_uri_query = "
						SELECT
							um.uri
						FROM
							" . TABLE_CEON_URI_MAPPINGS . " um
						WHERE
							um.main_page = '" . zen_db_input(zen_db_prepare_input(
								$_GET['main_page'])) . "'
						AND
							um.associated_db_id IS NULL
						AND
							um.query_string_parameters IS NULL
						AND
							um.language_id = '" . (int) $_SESSION['languages_id'] . "'
						AND
							um.current_uri = '1';";
					
					$redirection_uri_result = $db->Execute($redirection_uri_query);
					
					if (!$redirection_uri_result->EOF) {
						$redirection_uri = $redirection_uri_result->fields['uri'];
						
						// Perform a sanity check to make sure that the URI to be redirected to
						// isn't simply the current URI 
						if ($request_uri != $redirection_uri) {
							// Fine to redirect, remove unnecessary page identifier from query
							// string
							unset($_GET['main_page']);
						} else {
							// Don't redirect to the same address!
							$redirection_uri = null;
						}
					}
				} else if (is_null($associated_db_id) && !is_null($query_string_to_match) &&
						$_GET['main_page'] == FILENAME_DEFAULT) {
					// Build the canonical URI for a manufacturer page or a filtered page
					if (isset($_GET['manufacturers_id'])) {
						$ceon_uri_mapping_canonical_uri = HTTP_SERVER . DIR_WS_CATALOG .
							'index.php?main_page=index&manufacturers_id=' .
							$_GET['manufacturers_id'];
					} else if (isset($_GET['typefilter']) && $_GET['typefilter'] != '' &&
							isset($_GET[$_GET['typefilter'] . '_id']) &&
							$_GET[$_GET['typefilter'] . '_id'] != '') {
						$ceon_uri_mapping_canonical_uri = HTTP_SERVER . DIR_WS_CATALOG .
							'index.php?main_page=index&typefilter=' . $_GET['typefilter'] . '&' .
							$_GET['typefilter'] . '_id=' . $_GET[$_GET['typefilter'] . '_id'];
					}
				}
				
				if (!is_null($redirection_uri)) {
					require_once(DIR_FS_CATALOG . DIR_WS_FUNCTIONS .
						'ceon_uri_mapping_functions.php');
					
					$query_string = ceon_uri_mapping_buildQueryString();
					
					$redirection_uri .= $query_string;
					
					header("HTTP/1.1 301 Moved Permanently");
					header("Location: " . $redirection_uri);
					zen_exit();
				}
			}
		}
	}
}

?>